/* 
	Written by Robert Nyman, http://www.robertnyman.com
	Inspired by http://www.insideria.com/2008/03/air-api-performing-updates-in-1.html, and then modified
*/	
var AIRUpdater = function () {
	
	// This is only used for the filename of the installer
	var applicationName = "FlyCast.air";

	/*
		URL to go to to check the value of the <version> and <releasenotes>
		The value in version is compared to the one in the applications XML setup file,
		and an update dialog is triggered if the <version> in the below XML file is higher
		From there on, suggested alternatives for the end user is to 
		start an update or cancel it for the moment
		
		Suggested XML structure:
		
		<?xml version="1.0" encoding="UTF-8"?>
		<application>
			<latestversion>0.7</latestversion>
			<releasenotes>Added automatic update downloading feature.</releasenotes>
		</application>
	*/
	var latestVersionCheckUrl = "air/badge/versions/versionControl.xml";

	/*
		Change this URL to the download URL of your AIR app. Version number 
		and ".air" extension will automatically be added; version taken 
		from the XML value found in the latestVersionCheckUrl page
	*/	
	var stream = null;
	var updateFile = null;
	
	
	//These variables are used by the object
	var updateAvailable = null;
	var updateAvailableDialog = null;
	var latestVersion = 0;
	var applicationVersion = 0;
	var releaseNotes = null;
	var releaseNotesText = "";
	
	
	var getApplicationVersion = function () {
		// This will get the version of the currently installed application
		var appXML = air.NativeApplication.nativeApplication.applicationDescriptor;
		var xmlObject = new DOMParser().parseFromString(appXML, "text/xml");
		applicationVersion = parseFloat(xmlObject.getElementsByTagName('version')[0].firstChild.nodeValue);
	};
	
	var getLatestVersion = function () {
		/* 
			Checks for what the latest available version is 
			from the URL specified in latestVersionCheckUrl
		*/	
		var XMLHttp = new XMLHttpRequest();
		XMLHttp.onreadystatechange = function () {
			if (XMLHttp.readyState === 4) {
				var response = XMLHttp.responseXML;

				var releaseNotesNode = response.getElementsByTagName("releasenotes")[0];
				/*
					Adds a reference to a releaseNote for the latest version, 
					IF a <releasenotes> node exists
				*/
				if (typeof releaseNotesNode === "object" && releaseNotesNode.firstChild) {
					releaseNotesText = releaseNotesNode.firstChild.nodeValue;
				}
				var latestVersionNode = response.getElementsByTagName("latestversion")[0];
				/*
					Triggers a version comparison with the existing installed application, 
					IF a <latestversion> node exists
				*/	
				if (typeof latestVersionNode === "object" && latestVersionNode.firstChild) {
					latestVersion = parseFloat(latestVersionNode.firstChild.nodeValue, 10);
					compareVersions();
				}
			}
		};
		XMLHttp.open("GET", versionControlServer + latestVersionCheckUrl, true);
		XMLHttp.send(null);
	};
	
	var compareVersions = function () {
	
	    Debug("about to compare",1);
	    Debug("applicationVersion=" + applicationVersion,1);
	    Debug("latestVersion=" + latestVersion,1);
		if (applicationVersion > 0 && latestVersion > 0 && latestVersion > applicationVersion) {
			/*
				Here you should, for example, present an "Update available" to your
				end user, and give them the option to start the update
				The code below is just sample code:
			*/
			
			Debug("a new version is available",1);
			
			//turn off the application
			document.getElementById("root0").style.display = "none";
			
			updateAvailable = true;
			
			// Present release notes for the new version available
			document.getElementById("release-notes").innerHTML = releaseNotesText;
			
			// Add onclick event to start update button
			document.getElementById("update-application").onclick = initUpdateApplication;
			
			// Add onclick event to cancel update button
			document.getElementById("cancel-update").onclick = function () {
				document.getElementById("update-available-dialog").style.display = "none";
				document.getElementById("root0").style.display = "block";
			};
			
			// Show the update dialog to the end user
			document.getElementById("update-available-dialog").style.display = "block";
		}
		else
		{
		    Debug("no new version is available",1);
		    updateAvailable = false;
		}
	};
	
	var initUpdateApplication = function () {
		
		/* 
			The updating has started. Prefereably, you'd like to hide 
			or disable the start and cancel update buttons now
		*/
		document.getElementById("update-available-dialog").style.display = "none";
		document.getElementById("update-progress-dialog").style.display = "none";
		
		stream = new air.URLStream();
		
		/*
			This event is recommend to give the end user continuous 
			feedback about how the update goes
		*/
		stream.addEventListener(air.ProgressEvent.PROGRESS, updatingStatus);
		
		stream.addEventListener(air.Event.COMPLETE, updateApplication);
		
		// Note that the latest version number and ".air" extension is automatically added
		stream.load( new air.URLRequest(versionControlServer + "air/badge/versions/" + whiteLabelID  + "-" + latestVersion + ".air"));
	};
	
	var updatingStatus = function (e) {
		// This is example code to show updating status
		var percentage = Math.round((e.bytesLoaded / e.bytesTotal) * 100);
		document.getElementById("current-updating-status").innerHTML = percentage + "%";
	};
	
	updateApplication = function () {
		var ba = new air.ByteArray();
		stream.readBytes(ba, 0, stream.bytesAvailable);
		updateFile = air.File.applicationStorageDirectory.resolvePath(applicationName);
		fileStream = new air.FileStream();
		fileStream.addEventListener( air.Event.CLOSE, installUpdate );
		fileStream.openAsync(updateFile, air.FileMode.WRITE);
		fileStream.writeBytes(ba, 0, ba.length);
	 	fileStream.close();
	};
	
	var installUpdate = function () {
		var updater = new air.Updater();
		// Notice that the version name has to be present as a second parameter
		updater.update(updateFile, latestVersion.toString());
	};
	
	return {
		init : function () {
		    Debug("in init",1);
			getApplicationVersion();
			Debug("got the application version",1);
			getLatestVersion();
			Debug("got the latest version",1);
		}
	};
}();

//window.onload = AIRUpdater.init;
